@@ -37,10 +37,10 @@ from constant_sorrow.constants import (
     STRANGER_ALICE,
     UNKNOWN_VERSION,
     READY,
-    INVALIDATED
+    INVALIDATED,
+    NOT_SIGNED
 )
 from cryptography.hazmat.backends import default_backend
-from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurve
 from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.x509 import Certificate, NameOID, load_pem_x509_certificate
 from datetime import datetime
@@ -52,6 +52,7 @@ from json.decoder import JSONDecodeError
 from queue import Queue
 from random import shuffle
 from twisted.internet import reactor, stdio, threads
+from twisted.internet.defer import Deferred
 from twisted.internet.task import LoopingCall
 from twisted.logger import Logger
 from typing import Dict, Iterable, List, Tuple, Union, Optional, Sequence, Set
@@ -69,7 +70,6 @@ from nucypher.blockchain.eth.constants import ETH_ADDRESS_BYTE_LENGTH
 from nucypher.blockchain.eth.interfaces import BlockchainInterfaceFactory
 from nucypher.blockchain.eth.registry import BaseContractRegistry
 from nucypher.blockchain.eth.signers.software import Web3Signer
-from nucypher.blockchain.eth.token import WorkTracker
 from nucypher.characters.banners import ALICE_BANNER, BOB_BANNER, ENRICO_BANNER, URSULA_BANNER
 from nucypher.characters.base import Character, Learner
 from nucypher.characters.control.controllers import WebController
@@ -116,8 +116,6 @@ class Alice(Character, BlockchainPolicyAuthor):
 
                  # Ownership
                  checksum_address: str = None,
-                 client_password: str = None,
-                 cache_password: bool = False,
 
                  # M of N
                  m: int = None,
@@ -164,23 +162,22 @@ class Alice(Character, BlockchainPolicyAuthor):
 
         if is_me and not federated_only:  # TODO: #289
             blockchain = BlockchainInterfaceFactory.get_interface(provider_uri=self.provider_uri)
-            transacting_power = TransactingPower(account=self.checksum_address,
-                                                 password=client_password,
-                                                 cache=cache_password,
-                                                 signer=signer or Web3Signer(blockchain.client))
-
-            self._crypto_power.consume_power_up(transacting_power)
+            signer = signer or Web3Signer(blockchain.client)  # fallback to web3 provider by default for Alice.
+            self.transacting_power = TransactingPower(account=self.checksum_address, signer=signer)
+            self._crypto_power.consume_power_up(self.transacting_power)
             BlockchainPolicyAuthor.__init__(self,
+                                            domain=self.domain,
+                                            transacting_power=self.transacting_power,
                                             registry=self.registry,
                                             rate=rate,
-                                            duration_periods=duration_periods,
-                                            checksum_address=checksum_address)
+                                            duration_periods=duration_periods)
 
-        if is_me and controller:
-            self.make_cli_controller()
 
         self.log = Logger(self.__class__.__name__)
-        self.log.info(self.banner)
+        if is_me:
+            if controller:
+                self.make_cli_controller()
+            self.log.info(self.banner)
 
         self.active_policies = dict()
         self.revocation_kits = dict()
@@ -292,6 +289,14 @@ class Alice(Character, BlockchainPolicyAuthor):
             raise TypeError("For a federated policy, you must specify expiration (duration_periods don't count).")
         return base_payload
 
+    def _check_grant_requirements(self, policy):
+        """Called immediately before granting."""
+
+        # TODO: Remove when the time is right.
+        if policy.expiration > END_OF_POLICIES_PROBATIONARY_PERIOD:
+            raise self.ActorError(f"The requested duration for this policy (until {policy.expiration}) exceeds the "
+                                  f"probationary period ({END_OF_POLICIES_PROBATIONARY_PERIOD}).")
+
     def grant(self,
               bob: "Bob",
               label: bytes,
@@ -313,12 +318,7 @@ class Alice(Character, BlockchainPolicyAuthor):
                 self.remember_node(node=handpicked_ursula)
 
         policy = self.create_policy(bob=bob, label=label, **policy_params)
-
-        # TODO: Remove when the time is right.
-        if policy.expiration > END_OF_POLICIES_PROBATIONARY_PERIOD:
-            raise self.ActorError(f"The requested duration for this policy (until {policy.expiration}) exceeds the "
-                                  f"probationary period ({END_OF_POLICIES_PROBATIONARY_PERIOD}).")
-
+        self._check_grant_requirements(policy=policy)
         self.log.debug(f"Generated new policy proposal {policy} ... ")
 
         #
@@ -490,8 +490,18 @@ class Bob(Character):
         def __init__(self, evidence: List):
             self.evidence = evidence
 
-    def __init__(self, treasure_maps: Optional[Dict] = None, controller: bool = True, *args, **kwargs) -> None:
-        Character.__init__(self, known_node_class=Ursula, *args, **kwargs)
+    def __init__(self,
+                 is_me: bool = True,
+                 treasure_maps: Optional[Dict] = None,
+                 controller: bool = True,
+                 verify_node_bonding: bool = False,
+                 *args, **kwargs) -> None:
+
+        Character.__init__(self,
+                           is_me=is_me,
+                           known_node_class=Ursula,
+                           verify_node_bonding=verify_node_bonding,
+                           *args, **kwargs)
 
         if controller:
             self.make_cli_controller()
@@ -504,7 +514,8 @@ class Bob(Character):
         self._completed_work_orders = WorkOrderHistory()
 
         self.log = Logger(self.__class__.__name__)
-        self.log.info(self.banner)
+        if is_me:
+            self.log.info(self.banner)
 
     def get_card(self) -> 'Card':
         from nucypher.policy.identity import Card
@@ -798,15 +809,23 @@ class Bob(Character):
             return True, cfrags
 
     def retrieve(self,
+
+                 # Policy
                  *message_kits: UmbralMessageKit,
-                 alice_verifying_key: UmbralPublicKey,
+                 alice_verifying_key: Union[UmbralPublicKey, bytes],
                  label: bytes,
+
+                 # Source Authentication
                  enrico: "Enrico" = None,
+                 policy_encrypting_key: UmbralPublicKey = None,
+
+                 # Retrieval Behaviour
                  retain_cfrags: bool = False,
                  use_attached_cfrags: bool = False,
                  use_precedent_work_orders: bool = False,
-                 policy_encrypting_key: UmbralPublicKey = None,
-                 treasure_map: Union['TreasureMap', bytes] = None):
+                 treasure_map: Union['TreasureMap', bytes] = None
+
+                 ) -> List[bytes]:
 
         # Try our best to get an UmbralPublicKey from input
         alice_verifying_key = UmbralPublicKey.from_bytes(bytes(alice_verifying_key))
@@ -830,7 +849,12 @@ class Bob(Character):
             # self.treasure_maps[treasure_map.public_id()] = treasure_map # TODO: Can we?
         else:
             map_id = self.construct_map_id(alice_verifying_key, label)
-            treasure_map = self.treasure_maps[map_id]
+            try:
+                treasure_map = self.treasure_maps[map_id]
+            except KeyError:
+                # If the treasure map is not known, join the policy as part of retrieval.
+                self.join_policy(label=label, alice_verifying_key=alice_verifying_key)
+                treasure_map = self.treasure_maps[map_id]
 
         _unknown_ursulas, _known_ursulas, m = self.follow_treasure_map(treasure_map=treasure_map, block=True)
 
@@ -839,8 +863,7 @@ class Bob(Character):
 
         # Normalization
         for message in message_kits:
-            message.ensure_correct_sender(enrico=enrico,
-                                          policy_encrypting_key=policy_encrypting_key)
+            message.ensure_correct_sender(enrico=enrico, policy_encrypting_key=policy_encrypting_key)
 
         # Sanity check: If we're not using attached cfrags, we don't want a Capsule which has them.
         if not use_attached_cfrags and any(len(message.capsule) > 0 for message in message_kits):
@@ -958,7 +981,7 @@ class Bob(Character):
                 # Not enough matching nodes.  Fine, we'll just publish to the first few.
                 try:
                     # TODO: This is almost certainly happening in a test.  If it does happen in production, it's a bit of a problem.  Need to fix #2124 to mitigate.
-                    target_nodes = list(nodes._nodes.values())[0:6]
+                    target_nodes = list(nodes.values())[0:6]
                     return target_nodes
                 except IndexError:
                     raise self.NotEnoughNodes("There aren't enough nodes on the network to enact this policy.  Unless this is day one of the network and nodes are still getting spun up, something is bonkers.")
@@ -1012,12 +1035,15 @@ class Bob(Character):
 
 
 class Ursula(Teacher, Character, Worker):
+
     banner = URSULA_BANNER
     _alice_class = Alice
 
-    # TODO: Maybe this wants to be a registry, so that, for example,  NRN
-    # TLSHostingPower still can enjoy default status, but on a different class  NRN
-    _default_crypto_powerups = [SigningPower, DecryptingPower]
+    _default_crypto_powerups = [
+        SigningPower,
+        DecryptingPower,
+        # TLSHostingPower  # Still considered a default for Ursula, but needs the host context
+    ]
 
     _pruning_interval = 60  # seconds
 
@@ -1035,171 +1061,106 @@ class Ursula(Teacher, Character, Worker):
                  # Ursula
                  rest_host: str,
                  rest_port: int,
-                 domain: str = None,  # For now, serving and learning domain will be the same.
+                 domain: str,
+                 is_me: bool = True,
+
                  certificate: Certificate = None,
                  certificate_filepath: str = None,
+
                  db_filepath: str = None,
-                 is_me: bool = True,
                  interface_signature=None,
                  timestamp=None,
-                 availability_check: bool = False,
-                 prune_datastore: bool = True,
+                 availability_check: bool = False,  # TODO: Remove from init
 
                  # Blockchain
-                 decentralized_identity_evidence: bytes = constants.NOT_SIGNED,
-                 checksum_address: str = None,
-                 worker_address: str = None,  # TODO: deprecate, and rename to "checksum_address"
-                 block_until_ready: bool = True,
-                 # TODO: Must be true in order to set staker address - Allow for manual staker addr to be passed too!
-                 work_tracker: WorkTracker = None,
-                 commit_now: bool = True,
+                 checksum_address: ChecksumAddress = None,
+                 worker_address: ChecksumAddress = None,  # TODO: deprecate, and rename to "checksum_address"
                  client_password: str = None,
+                 decentralized_identity_evidence=NOT_SIGNED,
 
                  # Character
                  abort_on_learning_error: bool = False,
                  federated_only: bool = False,
-                 start_learning_now: bool = None,
                  crypto_power=None,
-                 tls_curve: EllipticCurve = None,
-                 known_nodes: Iterable = None,
+                 known_nodes: Iterable[Teacher] = None,
 
                  **character_kwargs
                  ) -> None:
 
-        #
-        # Character
-        #
-
-        if domain is None:
-            # TODO: Move defaults to configuration, Off character.
-            from nucypher.config.node import CharacterConfiguration
-            domain = CharacterConfiguration.DEFAULT_DOMAIN
-
-        if is_me:
-            # If we're federated only, we assume that all other nodes in our domain are as well.
-            self.set_federated_mode(federated_only)
-
         Character.__init__(self,
                            is_me=is_me,
                            checksum_address=checksum_address,
-                           start_learning_now=start_learning_now,
-                           federated_only=self._federated_only_instances,
-                           # TODO: 'Ursula' object has no attribute '_federated_only_instances' if an is_me Ursula is not inited prior to this moment  NRN
+                           federated_only=federated_only,
                            crypto_power=crypto_power,
                            abort_on_learning_error=abort_on_learning_error,
                            known_nodes=known_nodes,
                            domain=domain,
                            known_node_class=Ursula,
+                           include_self_in_the_state=True,
                            **character_kwargs)
 
         if is_me:
-            # Learner
-            self._start_learning_now = start_learning_now
 
-            # Self-Health Checks
+            # Operating Mode
+            self.known_node_class.set_federated_mode(federated_only)
+
+            # Health Checks
             self._availability_check = availability_check
             self._availability_tracker = AvailabilityTracker(ursula=self)
 
             # Datastore Pruning
-            self.__pruning_task = None
-            self._prune_datastore = prune_datastore
+            self.__pruning_task: Union[Deferred, None] = None
             self._datastore_pruning_task = LoopingCall(f=self.__prune_datastore)
 
-        #
-        # Ursula the Decentralized Worker (Self)
-        #
+            # Decentralized Worker
+            if not federated_only:
 
-        if is_me and not federated_only:  # TODO: #429
+                # TODO: Move to method
+                # Prepare a TransactingPower from worker node's transacting keys
+                transacting_power = TransactingPower(account=worker_address,
+                                                     password=client_password,
+                                                     signer=self.signer,
+                                                     cache=True)
+                self.transacting_power = transacting_power
+                self._crypto_power.consume_power_up(transacting_power)
 
-            # Prepare a TransactingPower from worker node's transacting keys
-            _transacting_power = TransactingPower(account=worker_address,
-                                                  password=client_password,
-                                                  signer=self.signer,
-                                                  cache=True)
+                # Use this power to substantiate the stamp
+                self.__substantiate_stamp()
+                decentralized_identity_evidence = self.__decentralized_identity_evidence
 
-            self.transacting_power = _transacting_power
-            self._crypto_power.consume_power_up(_transacting_power)
-            self._set_checksum_address(checksum_address)
+                try:
+                    Worker.__init__(self,
+                                    is_me=is_me,
+                                    domain=self.domain,
+                                    transacting_power=self.transacting_power,
+                                    registry=self.registry,
+                                    worker_address=worker_address)
+                except (Exception, self.WorkerError):
+                    # TODO: Do not announce self to "other nodes" until this init is finished.
+                    # It's not possible to finish constructing this node.
+                    self.stop(halt_reactor=False)
+                    raise
 
-            # Use this power to substantiate the stamp
-            self.substantiate_stamp()
-            self.log.debug(
-                f"Created decentralized identity evidence: {self.decentralized_identity_evidence[:10].hex()}")
-            decentralized_identity_evidence = self.decentralized_identity_evidence
+            self.rest_server = self._make_local_server(host=rest_host,
+                                                       port=rest_port,
+                                                       db_filepath=db_filepath,
+                                                       domain=domain)
 
-            try:
-                Worker.__init__(self,
-                                is_me=is_me,
-                                registry=self.registry,
-                                checksum_address=checksum_address,
-                                worker_address=worker_address,
-                                work_tracker=work_tracker,
-                                commit_now=commit_now,
-                                block_until_ready=block_until_ready)
-            except (Exception, self.WorkerError):  # FIXME
-                # TODO: Do not announce self to "other nodes" until this init is finished.
-                # It's not possible to finish constructing this node.
-                self.stop(halt_reactor=False)
-                raise
+            # Self-signed TLS certificate of self for Teacher.__init__
+            certificate_filepath = self._crypto_power.power_ups(TLSHostingPower).keypair.certificate_filepath
+            certificate = self._crypto_power.power_ups(TLSHostingPower).keypair.certificate
 
-        if not crypto_power or (TLSHostingPower not in crypto_power):
+            # only you can prevent forest fires
+            message = "THIS IS YOU: {}: {}".format(self.__class__.__name__, self)
+            self.log.info(message)
+            self.log.info(self.banner.format(self.nickname))
 
-            #
-            # Development Ursula
-            #
+        else:
+            # Stranger HTTP Server
+            # TODO: Use InterfaceInfo only
+            self.rest_server = ProxyRESTServer(rest_host=rest_host, rest_port=rest_port)
 
-            if is_me:
-                self.suspicious_activities_witnessed = {'vladimirs': [],
-                                                        'bad_treasure_maps': [],
-                                                        'freeriders': []}
-
-                # REST Server (Ephemeral Self-Ursula)
-                rest_app, datastore = make_rest_app(
-                    this_node=self,
-                    db_filepath=db_filepath,
-                    domain=domain,
-                )
-
-                # TLSHostingPower (Ephemeral Powers and Private Keys)
-                tls_hosting_keypair = HostingKeypair(curve=tls_curve, host=rest_host,
-                                                     checksum_address=self.checksum_address)
-                tls_hosting_power = TLSHostingPower(keypair=tls_hosting_keypair, host=rest_host)
-                self.rest_server = ProxyRESTServer(rest_host=rest_host, rest_port=rest_port,
-                                                   rest_app=rest_app, datastore=datastore,
-                                                   hosting_power=tls_hosting_power)
-
-            #
-            # Stranger-Ursula
-            #
-
-            else:
-
-                # TLSHostingPower
-                if certificate or certificate_filepath:
-                    tls_hosting_power = TLSHostingPower(host=rest_host,
-                                                        public_certificate_filepath=certificate_filepath,
-                                                        public_certificate=certificate)
-                else:
-                    tls_hosting_keypair = HostingKeypair(curve=tls_curve, host=rest_host, generate_certificate=False)
-                    tls_hosting_power = TLSHostingPower(host=rest_host, keypair=tls_hosting_keypair)
-
-                # REST Server
-                # Unless the caller passed a crypto power we'll make our own TLSHostingPower for this stranger.
-                self.rest_server = ProxyRESTServer(
-                    rest_host=rest_host,
-                    rest_port=rest_port,
-                    hosting_power=tls_hosting_power
-                )
-
-            # OK - Now we have a ProxyRestServer and a TLSHostingPower for some Ursula
-            self._crypto_power.consume_power_up(tls_hosting_power)  # Consume!
-
-        #
-        # Teacher (Verifiable Node)
-        #
-
-        certificate_filepath = self._crypto_power.power_ups(TLSHostingPower).keypair.certificate_filepath
-        certificate = self._crypto_power.power_ups(TLSHostingPower).keypair.certificate
+        # Teacher (All Modes)
         Teacher.__init__(self,
                          domain=domain,
                          certificate=certificate,
@@ -1208,15 +1169,43 @@ class Ursula(Teacher, Character, Worker):
                          timestamp=timestamp,
                          decentralized_identity_evidence=decentralized_identity_evidence)
 
-        if is_me:
-            self.known_nodes.record_fleet_state(additional_nodes_to_track=[self])  # Initial Impression
+    def __get_hosting_power(self, host: str) -> TLSHostingPower:
+        try:
+            # Pre-existing or injected power
+            tls_hosting_power = self._crypto_power.power_ups(TLSHostingPower)
+        except TLSHostingPower.not_found_error:
+            if self.keyring:
+                # Restore from TLS private key on-disk
+                tls_hosting_power = self.keyring.derive_crypto_power(TLSHostingPower, host=host)
+            else:
+                # Generate ephemeral private key ("Dev Mode")
+                tls_hosting_keypair = HostingKeypair(host=host,
+                                                     checksum_address=self.checksum_address,
+                                                     generate_certificate=True)
+                tls_hosting_power = TLSHostingPower(keypair=tls_hosting_keypair, host=host)
+            self._crypto_power.consume_power_up(tls_hosting_power)  # Consume!
+        return tls_hosting_power
 
-            message = "THIS IS YOU: {}: {}".format(self.__class__.__name__, self)
-            self.log.info(message)
-            self.log.info(self.banner.format(self.nickname))
-        else:
-            message = "Initialized Stranger {} | {}".format(self.__class__.__name__, self)
-            self.log.debug(message)
+    def _make_local_server(self, host, port, domain, db_filepath) -> ProxyRESTServer:
+        rest_app, datastore = make_rest_app(
+            this_node=self,
+            db_filepath=db_filepath,
+            domain=domain,
+        )
+        rest_server = ProxyRESTServer(rest_host=host,
+                                      rest_port=port,
+                                      rest_app=rest_app,
+                                      datastore=datastore,
+                                      hosting_power=self.__get_hosting_power(host=host))
+        return rest_server
+
+    def __substantiate_stamp(self):
+        transacting_power = self._crypto_power.power_ups(TransactingPower)
+        signature = transacting_power.sign_message(message=bytes(self.stamp))
+        self.__decentralized_identity_evidence = signature
+        self.__worker_address = transacting_power.account
+        message = f"Created decentralized identity evidence: {self.__decentralized_identity_evidence[:10].hex()}"
+        self.log.debug(message)
 
     def __prune_datastore(self) -> None:
         """Deletes all expired arrangements, kfrags, and treasure maps in the datastore."""
@@ -1254,24 +1243,34 @@ class Ursula(Teacher, Character, Worker):
                 self.log.debug(f"Pruned {result} treasure maps.")
 
     def __preflight(self) -> None:
-        """Called immediately before running services"""
+        """Called immediately before running services
+        If an exception is raised, Ursula startup will be interrupted.
+
+        """
         validate_worker_ip(worker_ip=self.rest_interface.host)
 
     def run(self,
             emitter: StdoutEmitter = None,
             discovery: bool = True,  # TODO: see below
-            availability: bool = True,
+            availability: bool = False,
             worker: bool = True,
             pruning: bool = True,
             interactive: bool = False,
             hendrix: bool = True,
             start_reactor: bool = True,
             prometheus_config: 'PrometheusMetricsConfig' = None,
-            preflight: bool = True
+            preflight: bool = True,
+            block_until_ready: bool = True,
+            eager: bool = False
             ) -> None:
 
         """Schedule and start select ursula services, then optionally start the reactor."""
 
+        # Connect to Provider
+        if not self.federated_only:
+            if not BlockchainInterfaceFactory.is_interface_initialized(provider_uri=self.provider_uri):
+                BlockchainInterfaceFactory.initialize_interface(provider_uri=self.provider_uri)
+
         if preflight:
             self.__preflight()
 
@@ -1283,23 +1282,27 @@ class Ursula(Teacher, Character, Worker):
             emitter.message(f"Starting services", color='yellow')
 
         if pruning:
-            self.__pruning_task = self._datastore_pruning_task.start(interval=self._pruning_interval, now=True)
+            self.__pruning_task = self._datastore_pruning_task.start(interval=self._pruning_interval, now=eager)
             if emitter:
                 emitter.message(f"✓ Database Pruning", color='green')
 
-        # TODO: Startup node discovery here with the rest of Ursula's services.
-        # if discovery and not self.lonely:
-        #     self.start_learning_loop(now=self._start_learning_now)
-        #     if emitter:
-        #         emitter.message(f"✓ Node Discovery - {self.domain}", color='green')
+        if discovery and not self.lonely:
+            self.start_learning_loop(now=eager)
+            if emitter:
+                emitter.message(f"✓ Node Discovery ({self.domain.capitalize()})", color='green')
 
-        if self._availability_check and availability:
-            self._availability_tracker.start(now=False)  # wait...
+        if self._availability_check or availability:
+            self._availability_tracker.start(now=eager)
             if emitter:
                 emitter.message(f"✓ Availability Checks", color='green')
 
         if worker and not self.federated_only:
-            self.work_tracker.start(commit_now=True)  # requirement_func=self._availability_tracker.status)  # TODO: #2277
+            if block_until_ready:
+                # Sets (staker's) checksum address; Prevent worker startup before bonding
+                self.block_until_ready()
+            self.stakes.checksum_address = self.checksum_address
+            self.stakes.refresh()
+            self.work_tracker.start(commit_now=eager)  # requirement_func=self._availability_tracker.status)  # TODO: #2277
             if emitter:
                 emitter.message(f"✓ Work Tracking", color='green')
 
@@ -1310,7 +1313,6 @@ class Ursula(Teacher, Character, Worker):
         if prometheus_config:
             # Locally scoped to prevent import without prometheus explicitly installed
             from nucypher.utilities.prometheus.metrics import start_prometheus_exporter
-            # TODO: Integrate with Hendrix TLS Deploy?
             start_prometheus_exporter(ursula=self, prometheus_config=prometheus_config)
             if emitter:
                 emitter.message(f"✓ Prometheus Exporter", color='green')
@@ -1340,7 +1342,7 @@ class Ursula(Teacher, Character, Worker):
                     emitter.message(f"{e.__class__.__name__} {e}", color='red', bold=True)
                 raise  # Crash :-(
 
-        elif start_reactor:  # ... without hendrix
+        if start_reactor:  # ... without hendrix
             reactor.run()  # <--- Blocking Call (Reactor)
 
     def stop(self, halt_reactor: bool = False) -> None:
@@ -1392,17 +1394,11 @@ class Ursula(Teacher, Character, Worker):
         deployer = self._crypto_power.power_ups(TLSHostingPower).get_deployer(rest_app=self.rest_app, port=port)
         return deployer
 
-    def rest_server_certificate(self):
-        return self._crypto_power.power_ups(TLSHostingPower).keypair.certificate
-
     def __bytes__(self):
 
         version = self.TEACHER_VERSION.to_bytes(2, "big")
         interface_info = VariableLengthBytestring(bytes(self.rest_interface))
-
-        certificate = self.rest_server_certificate()
-        cert_vbytes = VariableLengthBytestring(certificate.public_bytes(Encoding.PEM))
-
+        certificate_vbytes = VariableLengthBytestring(self.certificate.public_bytes(Encoding.PEM))
         as_bytes = bytes().join((version,
                                  self.canonical_public_address,
                                  bytes(VariableLengthBytestring(self.domain.encode('utf-8'))),
@@ -1411,7 +1407,7 @@ class Ursula(Teacher, Character, Worker):
                                  bytes(VariableLengthBytestring(self.decentralized_identity_evidence)),  # FIXME: Fixed length doesn't work with federated
                                  bytes(self.public_keys(SigningPower)),
                                  bytes(self.public_keys(DecryptingPower)),
-                                 bytes(cert_vbytes),
+                                 bytes(certificate_vbytes),  # TLSHostingPower
                                  bytes(interface_info))
                                 )
         return as_bytes
@@ -1447,6 +1443,18 @@ class Ursula(Teacher, Character, Worker):
         seed_uri = f'{seednode_metadata.checksum_address}@{seednode_metadata.rest_host}:{seednode_metadata.rest_port}'
         return cls.from_seed_and_stake_info(seed_uri=seed_uri, *args, **kwargs)
 
+    @classmethod
+    def seednode_for_network(cls, network: str) -> 'Ursula':
+        """Returns a default seednode ursula for a given network."""
+        try:
+            url = RestMiddleware.TEACHER_NODES[network][0]
+        except KeyError:
+            raise ValueError(f'"{network}" is not a known network.')
+        except IndexError:
+            raise ValueError(f'No default seednodes available for "{network}".')
+        ursula = cls.from_seed_and_stake_info(seed_uri=url)
+        return ursula
+
     @classmethod
     def from_teacher_uri(cls,
                          federated_only: bool,
@@ -1500,7 +1508,7 @@ class Ursula(Teacher, Character, Worker):
         #
 
         # Parse node URI
-        host, port, checksum_address = parse_node_uri(seed_uri)
+        host, port, staker_address = parse_node_uri(seed_uri)
 
         # Fetch the hosts TLS certificate and read the common name
         try:
@@ -1526,19 +1534,18 @@ class Ursula(Teacher, Character, Worker):
         )
 
         # Check the node's stake (optional)
-        if minimum_stake > 0 and not federated_only:
+        if minimum_stake > 0 and staker_address and not federated_only:
             staking_agent = ContractAgency.get_agent(StakingEscrowAgent, registry=registry)
-            seednode_stake = staking_agent.get_locked_tokens(staker_address=checksum_address)
+            seednode_stake = staking_agent.get_locked_tokens(staker_address=staker_address)
             if seednode_stake < minimum_stake:
-                raise Learner.NotATeacher(
-                    f"{checksum_address} is staking less than the specified minimum stake value ({minimum_stake}).")
+                raise Learner.NotATeacher(f"{staker_address} is staking less than the specified minimum stake value ({minimum_stake}).")
 
         # OK - everyone get out
         temp_node_storage.forget()
         return potential_seed_node
 
     @classmethod
-    def payload_splitter(cls, splittable, partial=False):
+    def payload_splitter(cls, splittable, partial: bool = False):
         splitter = BytestringKwargifier(
             _receiver=cls.from_processed_bytes,
             _partial_receiver=NodeSprout,
@@ -1546,7 +1553,10 @@ class Ursula(Teacher, Character, Worker):
             domain=VariableLengthBytestring,
             timestamp=(int, 4, {'byteorder': 'big'}),
             interface_signature=Signature,
-            decentralized_identity_evidence=VariableLengthBytestring,  # FIXME: Fixed length doesn't work with federated. It was LENGTH_ECDSA_SIGNATURE_WITH_RECOVERY,
+
+            # FIXME: Fixed length doesn't work with federated. It was LENGTH_ECDSA_SIGNATURE_WITH_RECOVERY,
+            decentralized_identity_evidence=VariableLengthBytestring,
+
             verifying_key=(UmbralPublicKey, PUBLIC_KEY_LENGTH),
             encrypting_key=(UmbralPublicKey, PUBLIC_KEY_LENGTH),
             certificate=(load_pem_x509_certificate, VariableLengthBytestring, {"backend": default_backend()}),
@@ -1733,19 +1743,25 @@ class Enrico(Character):
     _interface_class = EnricoInterface
     _default_crypto_powerups = [SigningPower]
 
-    def __init__(self, policy_encrypting_key=None, controller: bool = True, *args, **kwargs):
+    def __init__(self,
+                 is_me: bool = True,
+                 policy_encrypting_key: Optional[UmbralPublicKey] = None,
+                 controller: bool = True,
+                 *args, **kwargs):
+
         self._policy_pubkey = policy_encrypting_key
 
-        # Enrico never uses the blockchain, hence federated_only)
+        # Enrico never uses the blockchain (hence federated_only)
         kwargs['federated_only'] = True
         kwargs['known_node_class'] = None
-        super().__init__(*args, **kwargs)
+        super().__init__(is_me=is_me, *args, **kwargs)
 
         if controller:
             self.make_cli_controller()
 
         self.log = Logger(f'{self.__class__.__name__}-{bytes(self.public_keys(SigningPower)).hex()[:6]}')
-        self.log.info(self.banner.format(policy_encrypting_key))
+        if is_me:
+            self.log.info(self.banner.format(policy_encrypting_key))
 
     def encrypt_message(self, plaintext: bytes) -> Tuple[UmbralMessageKit, Signature]:
         # TODO: #2107 Rename to "encrypt"
