@@ -17,17 +17,14 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 
 
 import click
-import maya
 import os
 from constant_sorrow.constants import NO_PASSWORD
-from datetime import timedelta
-from web3.main import Web3
 
 from nucypher.blockchain.eth.signers.software import ClefSigner
 from nucypher.characters.control.emitters import StdoutEmitter
 from nucypher.characters.control.interfaces import AliceInterface
-from nucypher.characters.lawful import Bob
 from nucypher.cli.actions.auth import get_client_password, get_nucypher_password
+from nucypher.cli.actions.collect import collect_bob_public_keys, collect_policy_parameters
 from nucypher.cli.actions.configure import (
     destroy_configuration,
     handle_missing_configuration_file,
@@ -35,6 +32,7 @@ from nucypher.cli.actions.configure import (
 )
 from nucypher.cli.actions.confirm import confirm_staged_grant
 from nucypher.cli.actions.select import select_client_account, select_config_file
+from nucypher.cli.actions.validate import validate_grant_command
 from nucypher.cli.commands.deploy import option_gas_strategy
 from nucypher.cli.config import group_general_config
 from nucypher.cli.options import (
@@ -62,13 +60,8 @@ from nucypher.cli.options import (
     option_lonely, option_max_gas_price
 )
 from nucypher.cli.painting.help import paint_new_installation_help
-from nucypher.cli.painting.help import (
-    paint_probationary_period_disclaimer,
-    enforce_probationary_period
-)
 from nucypher.cli.painting.policies import paint_single_card
 from nucypher.cli.types import EIP55_CHECKSUM_ADDRESS
-from nucypher.cli.types import GWEI
 from nucypher.cli.utils import make_cli_character, setup_emitter
 from nucypher.config.characters import AliceConfiguration
 from nucypher.config.constants import (
@@ -266,38 +259,24 @@ class AliceCharacterOptions:
         self.min_stake = min_stake
 
     def create_character(self, emitter, config_file, json_ipc, load_seednodes=True):
-
         config = self.config_options.create_config(emitter, config_file)
-
-        client_password = None
-        is_clef = ClefSigner.is_valid_clef_uri(self.config_options.signer_uri)
-        eth_password_is_needed = not config.federated_only and not self.hw_wallet and not config.dev_mode and not is_clef
-        if eth_password_is_needed:
-            if json_ipc:
-                client_password = os.environ.get(NUCYPHER_ENVVAR_ALICE_ETH_PASSWORD, NO_PASSWORD)
-                if client_password is NO_PASSWORD:
-                    message = f"--json-ipc implies the {NUCYPHER_ENVVAR_ALICE_ETH_PASSWORD} envvar must be set."
-                    click.BadOptionUsage(option_name='--json-ipc', message=message)
-            else:
-                client_password = get_client_password(checksum_address=config.checksum_address,
-                                                      envvar=NUCYPHER_ENVVAR_ALICE_ETH_PASSWORD)
-
         try:
             ALICE = make_cli_character(character_config=config,
                                        emitter=emitter,
                                        unlock_keyring=not config.dev_mode,
+                                       unlock_signer=not config.federated_only,
                                        teacher_uri=self.teacher_uri,
                                        min_stake=self.min_stake,
-                                       client_password=client_password,
                                        start_learning_now=load_seednodes,
-                                       lonely=self.config_options.lonely)
-
+                                       lonely=self.config_options.lonely,
+                                       json_ipc=json_ipc)
             return ALICE
         except NucypherKeyring.AuthenticationFailed as e:
             emitter.echo(str(e), color='red', bold=True)
             click.get_current_context().exit(1)
 
 
+
 group_character_options = group_options(
     AliceCharacterOptions,
     config_options=group_config_options,
@@ -413,7 +392,9 @@ def public_keys(general_config, character_options, config_file):
 def make_card(general_config, character_options, config_file, nickname):
     """Create a character card file for public key sharing"""
     emitter = setup_emitter(general_config)
-    ALICE = character_options.create_character(emitter, config_file, general_config.json_ipc, load_seednodes=False)
+    ALICE = character_options.create_character(emitter, config_file,
+                                               json_ipc=general_config.json_ipc,
+                                               load_seednodes=False)
     card = Card.from_character(ALICE)
     if nickname:
         card.nickname = nickname
@@ -430,7 +411,10 @@ def make_card(general_config, character_options, config_file, nickname):
 def derive_policy_pubkey(general_config, label, character_options, config_file):
     """Get a policy public key from a policy label."""
     emitter = setup_emitter(general_config)
-    ALICE = character_options.create_character(emitter, config_file, general_config.json_ipc, load_seednodes=False)
+    ALICE = character_options.create_character(emitter,
+                                               config_file,
+                                               json_ipc=general_config.json_ipc,
+                                               load_seednodes=False)
     return ALICE.controller.derive_policy_encrypting_key(label=label)
 
 
@@ -453,106 +437,69 @@ def grant(general_config,
           character_options,
           config_file,
           force):
-    """Create and enact an access policy for some Bob. """
+    """Create and enact an access policy for Bob."""
 
     # Setup
     emitter = setup_emitter(general_config)
-    ALICE = character_options.create_character(emitter, config_file, general_config.json_ipc)
+    ALICE = character_options.create_character(
+        emitter=emitter,
+        config_file=config_file,
+        json_ipc=general_config.json_ipc
+    )
+    validate_grant_command(
+        emitter=emitter,
+        alice=ALICE,
+        force=force,
+        bob=bob,
+        label=label,
+        rate=rate,
+        value=value,
+        expiration=expiration,
+        bob_encrypting_key=bob_encrypting_key,
+        bob_verifying_key=bob_verifying_key
+    )
 
-    # Policy option validation
-    if ALICE.federated_only:
-        if any((value, rate)):
-            message = "Can't use --value or --rate with a federated Alice."
-            raise click.BadOptionUsage(option_name="--value, --rate", message=message)
-    elif bool(value) and bool(rate):
-        raise click.BadOptionUsage(option_name="--rate", message="Can't use --value if using --rate")
+    # Collect
+    bob_public_keys = collect_bob_public_keys(
+        emitter=emitter,
+        force=force,
+        card_identifier=bob,
+        bob_encrypting_key=bob_encrypting_key,
+        bob_verifying_key=bob_verifying_key
+    )
 
-    # Grantee selection
-    if bob and any((bob_encrypting_key, bob_verifying_key)):
-        message = '--bob cannot be used with --bob-encrypting-key or --bob-veryfying key'
-        raise click.BadOptionUsage(option_name='--bob', message=message)
+    policy = collect_policy_parameters(
+            emitter=emitter,
+            alice=ALICE,
+            force=force,
+            bob_identifier=bob_public_keys.verifying_key[:8],
+            label=label,
+            m=m,
+            n=n,
+            rate=rate,
+            value=value,
+            expiration=expiration
+    )
 
-    if bob:
-        card = Card.load(identifier=bob)
-        if card.character is not Bob:
-            emitter.error('Grantee card is not a Bob.')
-            raise click.Abort
-        paint_single_card(emitter=emitter, card=card)
-        if not force:
-            click.confirm('Is this the correct grantee (Bob)?', abort=True)
-        bob_encrypting_key = card.encrypting_key.hex()
-        bob_verifying_key = card.verifying_key.hex()
-
-    # Interactive collection follows:
-    # TODO: Extricate to support modules
-    # - Disclaimer
-    # - Label
-    # - Expiration Date & Time
-    # - M of N
-    # - Policy Value (ETH)
-
-    # Policy Expiration
-    # TODO: Remove this line when the time is right.
-    paint_probationary_period_disclaimer(emitter)
-
-    # Label
-    if not label:
-        label = click.prompt(f'Enter label to grant Bob {bob_verifying_key[:8]}', type=click.STRING)
-
-    if not force and not expiration:
-        if ALICE.duration_periods:
-            # TODO: use a default in days or periods?
-            expiration = maya.now() + timedelta(days=ALICE.duration_periods)  # default
-            if not click.confirm(f'Use default policy duration (expires {expiration})?'):
-                expiration = click.prompt('Enter policy expiration datetime', type=click.DateTime())
-        else:
-            # No policy duration default default available; Go interactive
-            expiration = click.prompt('Enter policy expiration datetime', type=click.DateTime())
-
-    # TODO: Remove this line when the time is right.
-    enforce_probationary_period(emitter=emitter, expiration=expiration)
-
-    # Policy Threshold and Shares
-    if not n:
-        n = ALICE.n
-        if not force and not click.confirm(f'Use default value for N ({n})?', default=True):
-            n = click.prompt('Enter total number of shares (N)', type=click.INT)
-    if not m:
-        m = ALICE.m
-        if not force and not click.confirm(f'Use default value for M ({m})?', default=True):
-            m = click.prompt('Enter threshold (M)', type=click.IntRange(1, n))
-
-    # Policy Value
-    policy_value_provided = bool(value) or bool(rate)
-    if not ALICE.federated_only and not policy_value_provided:
-        rate = ALICE.default_rate  # TODO #1709 - Fine tuning and selection of default rates
-        if not force:
-            default_gwei = Web3.fromWei(rate, 'gwei')
-            prompt = "Confirm rate of {node_rate} gwei ({total_rate} gwei per period)?"
-            if not click.confirm(prompt.format(node_rate=default_gwei, total_rate=default_gwei*n), default=True):
-                interactive_rate = click.prompt('Enter rate per period in gwei', type=GWEI)
-                # TODO: Validate interactively collected rate (#1709)
-                click.confirm(prompt.format(node_rate=rate, total_rate=rate*n), default=True, abort=True)
-                rate = Web3.toWei(interactive_rate, 'gwei')
-
-    # Request
     grant_request = {
-        'bob_encrypting_key': bob_encrypting_key,
-        'bob_verifying_key': bob_verifying_key,
-        'label': label,
-        'm': m,
-        'n': n,
-        'expiration': expiration,
+        'bob_encrypting_key': bob_public_keys.encrypting_key,
+        'bob_verifying_key': bob_public_keys.verifying_key,
+        'label': policy.label,
+        'm': policy.m,
+        'n': policy.n,
+        'expiration': policy.expiration,
     }
     if not ALICE.federated_only:
-        if value:
-            grant_request['value'] = value
-        elif rate:
-            grant_request['rate'] = rate  # in wei
+        # These values can be 0
+        if policy.value is not None:
+            grant_request['value'] = policy.value
+        elif policy.rate is not None:
+            grant_request['rate'] = policy.rate  # in wei
 
+    # Grant
     if not force and not general_config.json_ipc:
-        confirm_staged_grant(emitter=emitter, grant_request=grant_request)
-    emitter.echo(f'Granting Access to {bob_verifying_key[:8]}', color='yellow')
+        confirm_staged_grant(emitter=emitter, grant_request=grant_request, federated=ALICE.federated_only)
+    emitter.echo(f'Granting Access to {bob_public_keys.verifying_key[:8]}', color='yellow')
     return ALICE.controller.grant(request=grant_request)
 
 
