@@ -40,8 +40,15 @@ from nucypher.cli.options import (
 )
 from nucypher.cli.painting.staking import paint_fee_rate_range
 from nucypher.cli.painting.status import paint_contract_status, paint_locked_tokens_status, paint_stakers
-from nucypher.cli.utils import connect_to_blockchain, get_registry, setup_emitter
+from nucypher.cli.utils import (
+    connect_to_blockchain,
+    get_registry,
+    setup_emitter,
+    retrieve_events,
+    parse_event_filters_into_argument_filters
+)
 from nucypher.config.constants import NUCYPHER_ENVVAR_PROVIDER_URI
+from nucypher.utilities.events import generate_events_csv_file
 
 
 class RegistryOptions:
@@ -71,6 +78,26 @@ group_registry_options = group_options(
     provider_uri=option_provider_uri(default=os.environ.get(NUCYPHER_ENVVAR_PROVIDER_URI)),
 )
 
+option_csv = click.option('--csv',
+                          help="Write event data to a CSV file using a default filename in the current directory",
+                          default=False,
+                          is_flag=True)
+option_csv_file = click.option('--csv-file',
+                               help="Write event data to the CSV file at specified filepath",
+                               type=click.Path(dir_okay=False))
+option_event_filters = click.option('--event-filter', '-f', 'event_filters',
+                                    help="Event filter of the form <name>=<value>",
+                                    multiple=True,
+                                    type=click.STRING,
+                                    default=[])
+
+option_from_block = click.option('--from-block',
+                                 help="Collect events from this block number; defaults to the block number of current period",
+                                 type=click.INT)
+option_to_block = click.option('--to-block',
+                               help="Collect events until this block number; defaults to 'latest' block number",
+                               type=click.INT)
+
 
 @click.group()
 def status():
@@ -114,21 +141,38 @@ def locked_tokens(general_config, registry_options, periods):
 @group_general_config
 @option_contract_name(required=False)
 @option_event_name
-@click.option('--from-block', help="Collect events from this block number", type=click.INT)
-@click.option('--to-block', help="Collect events until this block number", type=click.INT)
+@option_from_block
+@option_to_block
+@option_csv
+@option_csv_file
+@option_event_filters
 # TODO: Add options for number of periods in the past (default current period), or range of blocks
-# TODO: Add way to input additional event filters? (e.g., staker, etc)
-def events(general_config, registry_options, contract_name, from_block, to_block, event_name):
-    """Show events associated to NuCypher contracts."""
+def events(general_config, registry_options, contract_name, from_block, to_block, event_name, csv, csv_file, event_filters):
+    """Show events associated with NuCypher contracts."""
 
-    emitter, registry, blockchain = registry_options.setup(general_config=general_config)
+    if csv or csv_file:
+        if csv and csv_file:
+            raise click.BadOptionUsage(option_name='--event-filter',
+                                       message=f'Pass either --csv or --csv-file, not both.')
+
+        # ensure that event name is specified - different events would have different columns in the csv file
+        if csv_file and not all((event_name, contract_name)):
+            # TODO consider a single csv that just gets appended to for each event
+            #  - each appended event adds their column names first
+            #  - single report-type functionality, see #2561
+            raise click.BadOptionUsage(option_name='--csv-file, --event-name, --contract_name',
+                                       message='--event-name and --contract-name must be specified when outputting to '
+                                               'specific file using --csv-file; alternatively use --csv')
     if not contract_name:
         if event_name:
             raise click.BadOptionUsage(option_name='--event-name', message='--event-name requires --contract-name')
+        # FIXME should we force a contract name to be specified?
         contract_names = [STAKING_ESCROW_CONTRACT_NAME, POLICY_MANAGER_CONTRACT_NAME]
     else:
         contract_names = [contract_name]
 
+    emitter, registry, blockchain = registry_options.setup(general_config=general_config)
+
     if from_block is None:
         # by default, this command only shows events of the current period
         last_block = blockchain.client.block_number
@@ -139,19 +183,47 @@ def events(general_config, registry_options, contract_name, from_block, to_block
                                                       latest_block=last_block)
     if to_block is None:
         to_block = 'latest'
+    else:
+        # validate block range
+        if from_block > to_block:
+            raise click.BadOptionUsage(option_name='--to-block, --from-block',
+                                       message=f'Invalid block range provided, '
+                                               f'from-block ({from_block}) > to-block ({to_block})')
 
-    # TODO: additional input validation for block numbers
-    emitter.echo(f"Showing events from block {from_block} to {to_block}")
+    # event argument filters
+    argument_filters = None
+    if event_filters:
+        try:
+            argument_filters = parse_event_filters_into_argument_filters(event_filters)
+        except ValueError as e:
+            raise click.BadOptionUsage(option_name='--event-filter',
+                                       message=f'Event filter must be specified as name-value pairs of '
+                                               f'the form `<name>=<value>` - {str(e)}')
+
+    emitter.echo(f"Retrieving events from block {from_block} to {to_block}")
     for contract_name in contract_names:
-        title = f" {contract_name} Events ".center(40, "-")
-        emitter.echo(f"\n{title}\n", bold=True, color='green')
         agent = ContractAgency.get_agent_by_contract_name(contract_name, registry)
+        if event_name and event_name not in agent.events.names:
+            raise click.BadOptionUsage(option_name='--event-name, --contract_name',
+                                       message=f'{contract_name} contract does not have an event named {event_name}')
+
+        title = f" {agent.contract_name} Events ".center(40, "-")
+        emitter.echo(f"\n{title}\n", bold=True, color='green')
         names = agent.events.names if not event_name else [event_name]
         for name in names:
-            emitter.echo(f"{name}:", bold=True, color='yellow')
-            event_method = agent.events[name]
-            for event_record in event_method(from_block=from_block, to_block=to_block):
-                emitter.echo(f"  - {event_record}")
+            # csv output file - one per (contract_name, event_name) pair
+            csv_output_file = csv_file
+            if csv or csv_output_file:
+                if not csv_output_file:
+                    csv_output_file = generate_events_csv_file(contract_name=agent.contract_name, event_name=name)
+
+            retrieve_events(emitter=emitter,
+                            agent=agent,
+                            event_name=name,  # None is fine - just means all events
+                            from_block=from_block,
+                            to_block=to_block,
+                            argument_filters=argument_filters,
+                            csv_output_file=csv_output_file)
 
 
 @status.command(name='fee-range')
