@@ -100,8 +100,6 @@ PROMPT_STAKE_INCREASE_VALUE = "Enter stake value in NU (up to {upper_limit})"
 
 SUCCESSFUL_STAKE_INCREASE = 'Successfully increased stake'
 
-PREALLOCATION_STAKE_ADVISORY = "Beneficiary {client_account} will use preallocation contract {staking_address} to stake."
-
 NO_STAKING_ACCOUNTS = "No staking accounts found."
 
 SELECT_STAKING_ACCOUNT_INDEX = "Select index of staking account"
@@ -118,9 +116,15 @@ POST_STAKING_ADVICE = """
 View your stakes by running 'nucypher stake list'
 or set your Ursula worker node address by running 'nucypher stake bond-worker'.
 
-See https://docs.nucypher.com/en/latest/guides/network_node/staking_guide.html
+See https://docs.nucypher.com/en/latest/staking/running_a_worker.html
 """
 
+
+#
+# Events
+#
+CONFIRM_OVERWRITE_EVENTS_CSV_FILE = "Overwrite existing CSV events file - {csv_file}?"
+
 #
 # Minting
 #
@@ -169,15 +173,6 @@ SUCCESSFUL_DISABLE_WIND_DOWN = 'Successfully disabled winding down for {staking_
 # Restaking
 #
 
-RESTAKING_LOCK_AGREEMENT = """
-By enabling the re-staking lock for {staking_address}, you are committing to automatically
-re-stake all rewards until a future period.  You will not be able to disable re-staking until {release_period}.
-"""
-
-CONFIRM_RESTAKING_LOCK = "Confirm enable re-staking lock for staker {staking_address} until {release_period}?"
-
-SUCCESSFUL_ENABLE_RESTAKE_LOCK = 'Successfully enabled re-staking lock for {staking_address} until {lock_until}'
-
 RESTAKING_AGREEMENT = """
 By enabling the re-staking for {staking_address}, all staking rewards will be automatically added to your existing stake.
 """
@@ -425,8 +420,6 @@ DISPLAY_SENDER_TOKEN_BALANCE_BEFORE_TRANSFER = "Deployer NU balance: {token_bala
 
 PROMPT_FOR_ALLOCATION_DATA_FILEPATH = "Enter allocations data filepath"
 
-SUCCESSFUL_SAVE_BATCH_DEPOSIT_RECEIPTS = "Saved batch deposits receipts to {receipts_filepath}"
-
 SUCCESSFUL_SAVE_DEPLOY_RECEIPTS = "Saved deployment receipts to {receipts_filepath}"
 
 SUCCESSFUL_REGISTRY_CREATION = 'Wrote to registry {registry_outfile}'
