@@ -17,20 +17,21 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 
 
 import os
-from constant_sorrow.constants import (
-    UNINITIALIZED_CONFIGURATION
-)
+from constant_sorrow.constants import UNINITIALIZED_CONFIGURATION
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurve
 from cryptography.x509 import Certificate
 from tempfile import TemporaryDirectory
 
-from nucypher.blockchain.eth.networks import NetworksInventory
 from nucypher.blockchain.eth.actors import StakeHolder
-from nucypher.blockchain.eth.signers import Signer
-from nucypher.config.constants import DEFAULT_CONFIG_ROOT
+from nucypher.config.base import CharacterConfiguration
+from nucypher.config.constants import (
+    DEFAULT_CONFIG_ROOT,
+    NUCYPHER_ENVVAR_WORKER_ETH_PASSWORD,
+    NUCYPHER_ENVVAR_ALICE_ETH_PASSWORD,
+    NUCYPHER_ENVVAR_BOB_ETH_PASSWORD
+)
 from nucypher.config.keyring import NucypherKeyring
-from nucypher.config.node import CharacterConfiguration
 
 
 class UrsulaConfiguration(CharacterConfiguration):
@@ -42,10 +43,10 @@ class UrsulaConfiguration(CharacterConfiguration):
     DEFAULT_REST_PORT = 9151
     DEFAULT_DEVELOPMENT_REST_HOST = '127.0.0.1'
     DEFAULT_DEVELOPMENT_REST_PORT = 10151
-    __DEFAULT_TLS_CURVE = ec.SECP384R1
     DEFAULT_DB_NAME = f'{NAME}.db'
     DEFAULT_AVAILABILITY_CHECKS = False
     LOCAL_SIGNERS_ALLOWED = True
+    SIGNER_ENVVAR = NUCYPHER_ENVVAR_WORKER_ETH_PASSWORD
 
     def __init__(self,
                  rest_host: str = None,
@@ -53,7 +54,6 @@ class UrsulaConfiguration(CharacterConfiguration):
                  dev_mode: bool = False,
                  db_filepath: str = None,
                  rest_port: int = None,
-                 tls_curve: EllipticCurve = None,
                  certificate: Certificate = None,
                  availability_check: bool = None,
                  *args, **kwargs) -> None:
@@ -70,7 +70,6 @@ class UrsulaConfiguration(CharacterConfiguration):
 
         self.rest_port = rest_port
         self.rest_host = rest_host
-        self.tls_curve = tls_curve or self.__DEFAULT_TLS_CURVE
         self.certificate = certificate
         self.db_filepath = db_filepath or UNINITIALIZED_CONFIGURATION
         self.worker_address = worker_address
@@ -101,7 +100,6 @@ class UrsulaConfiguration(CharacterConfiguration):
     def dynamic_payload(self) -> dict:
         payload = dict(
             network_middleware=self.network_middleware,
-            tls_curve=self.tls_curve,  # TODO: Needs to be in static payload with [str -> curve] mapping
             certificate=self.certificate,
             interface_signature=self.interface_signature,
             timestamp=None
@@ -134,7 +132,6 @@ class UrsulaConfiguration(CharacterConfiguration):
                                         encrypting=True,
                                         rest=True,
                                         host=self.rest_host,
-                                        curve=self.tls_curve,
                                         checksum_address=self.worker_address,
                                         **generation_kwargs)
         return keyring
@@ -160,6 +157,8 @@ class AliceConfiguration(CharacterConfiguration):
     DEFAULT_STORE_POLICIES = True
     DEFAULT_STORE_CARDS = True
 
+    SIGNER_ENVVAR = NUCYPHER_ENVVAR_ALICE_ETH_PASSWORD
+
     _CONFIG_FIELDS = (
         *CharacterConfiguration._CONFIG_FIELDS,
         'store_policies',
@@ -215,6 +214,7 @@ class BobConfiguration(CharacterConfiguration):
     DEFAULT_CONTROLLER_PORT = 7151
     DEFFAULT_STORE_POLICIES = True
     DEFAULT_STORE_CARDS = True
+    SIGNER_ENVVAR = NUCYPHER_ENVVAR_BOB_ETH_PASSWORD
 
     _CONFIG_FIELDS = (
         *CharacterConfiguration._CONFIG_FIELDS,
@@ -258,6 +258,7 @@ class FelixConfiguration(CharacterConfiguration):
     DEFAULT_REST_HOST = '127.0.0.1'
     __DEFAULT_TLS_CURVE = ec.SECP384R1
 
+
     def __init__(self,
                  db_filepath: str = None,
                  rest_host: str = None,
@@ -298,6 +299,11 @@ class StakeHolderConfiguration(CharacterConfiguration):
     NAME = 'stakeholder'
     CHARACTER_CLASS = StakeHolder
 
+    _CONFIG_FIELDS = (
+        *CharacterConfiguration._CONFIG_FIELDS,
+        'provider_uri'
+    )
+
     def __init__(self, checksum_addresses: set = None, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.checksum_addresses = checksum_addresses
@@ -310,8 +316,6 @@ class StakeHolderConfiguration(CharacterConfiguration):
                        poa=self.poa,
                        light=self.is_light,
                        domain=self.domain,
-                       # TODO: Move empty collection casting to base
-                       checksum_addresses=self.checksum_addresses or list(),
                        signer_uri=self.signer_uri,
                        worker_data=self.worker_data
                        )
@@ -322,12 +326,10 @@ class StakeHolderConfiguration(CharacterConfiguration):
 
     @property
     def dynamic_payload(self) -> dict:
-        testnet = self.domain != NetworksInventory.MAINNET
-        signer = Signer.from_signer_uri(self.signer_uri, testnet=testnet)
-        payload = dict(registry=self.registry, signer=signer)
+        payload = dict(registry=self.registry, signer=self.signer)
         return payload
 
-    def __setup_node_storage(self, node_storage=None) -> None:
+    def _setup_node_storage(self, node_storage=None) -> None:
         pass
 
     @classmethod
