@@ -15,13 +15,19 @@ You should have received a copy of the GNU Affero General Public License
 along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+
 import binascii
 import os
 import uuid
 import weakref
 from bytestring_splitter import BytestringSplitter
 from constant_sorrow import constants
-from constant_sorrow.constants import FLEET_STATES_MATCH, NO_BLOCKCHAIN_CONNECTION, NO_KNOWN_NODES, RELAX
+from constant_sorrow.constants import (
+    FLEET_STATES_MATCH,
+    NO_BLOCKCHAIN_CONNECTION,
+    NO_KNOWN_NODES,
+    RELAX
+)
 from datetime import datetime, timedelta
 from flask import Flask, Response, jsonify, request
 from mako import exceptions as mako_exceptions
@@ -170,10 +176,10 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
     def all_known_nodes():
         headers = {'Content-Type': 'application/octet-stream'}
         if this_node._learning_deferred is not RELAX and not this_node._learning_task.running:
-            # TODO: Is this every something we don't want to do?
+            # Learn when learned about
             this_node.start_learning_loop()
 
-        if this_node.known_nodes.checksum is NO_KNOWN_NODES:
+        if not this_node.known_nodes:
             return Response(b"", headers=headers, status=204)
 
         known_nodes_bytestring = this_node.bytestring_of_known_nodes()
@@ -232,7 +238,7 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
             cleartext = this_node.verify_from(alice, policy_message_kit, decrypt=True)
         except InvalidSignature:
             # TODO: Perhaps we log this?  Essentially 355.
-            return Response(status_code=400)
+            return Response("Invalid Signature", status_code=400)
 
         if not this_node.federated_only:
             # This splitter probably belongs somewhere canonical.
@@ -246,23 +252,23 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
             except TimeExhausted:
                 # Alice didn't pay.  Return response with that weird status code.
                 this_node.suspicious_activities_witnessed['freeriders'].append((alice, f"No transaction matching {tx}."))
-                return Response(status=402)
+                return Response(f"No paid transaction matching {tx} for this node", status=402)
 
             this_node_has_been_arranged = this_node.checksum_address in arranged_addresses
             if not this_node_has_been_arranged:
                 this_node.suspicious_activities_witnessed['freeriders'].append((alice, f"The transaction {tx} does not list me as a Worker - it lists {arranged_addresses}."))
-                return Response(status=402)
+                return Response(f"This node was not listed as servicing the policy in transaction {tx}", status=402)
         else:
             _tx = NO_BLOCKCHAIN_CONNECTION
             kfrag_bytes = cleartext
         kfrag = KFrag.from_bytes(kfrag_bytes)
 
         if not kfrag.verify(signing_pubkey=alices_verifying_key):
-            raise InvalidSignature("{} is invalid".format(kfrag))
+            return Response(f"Signature on {kfrag} is invalid", status=403)
 
         with datastore.describe(PolicyArrangement, id_as_hex, writeable=True) as policy_arrangement:
             if not policy_arrangement.alice_verifying_key == alice.stamp.as_umbral_pubkey():
-                raise alice.SuspiciousActivity
+                return Response("Policy arrangement's signing key does not match sender's", status=403)
             policy_arrangement.kfrag = kfrag
 
         # TODO: Sign the arrangement here.  #495
@@ -422,30 +428,28 @@ def _make_rest_app(datastore: Datastore, this_node, domain: str, log: Logger) ->
 
     @rest_app.route('/status/', methods=['GET'])
     def status():
-        if request.args.get('json'):
-            payload = this_node.abridged_node_details(raise_invalid=False)
-            response = jsonify(payload)
-            return response
+
+        return_json = request.args.get('json') == 'true'
+        omit_known_nodes = request.args.get('omit_known_nodes') == 'true'
+
+        status_info = this_node.status_info(raise_invalid=False,
+                                            omit_known_nodes=omit_known_nodes)
+
+        if return_json:
+            return jsonify(status_info)
 
         else:
             headers = {"Content-Type": "text/html", "charset": "utf-8"}
-            previous_states = list(reversed(this_node.known_nodes.states.values()))[:5]
-            # Mature every known node before rendering.
-            for node in this_node.known_nodes:
-                node.mature()
 
             try:
-                content = status_template.render(this_node=this_node,
-                                                 known_nodes=this_node.known_nodes,
-                                                 previous_states=previous_states,
-                                                 domain=domain,
-                                                 version=nucypher.__version__,
-                                                 checksum_address=this_node.checksum_address)
+                content = status_template.render(status_info)
+
             except Exception as e:
                 text_error = mako_exceptions.text_error_template().render()
                 html_error = mako_exceptions.html_error_template().render()
                 log.debug("Template Rendering Exception:\n" + text_error)
                 return Response(response=html_error, headers=headers, status=500)
+
             return Response(response=content, headers=headers)
 
     return rest_app
